# -*- coding:utf-8 -*-

# author: Cone
# datetime: 2022/3/20 下午4:26
# software: PyCharm

import signal


def on_sys_interrupt(func):
    signal.signal(signal.SIGTERM, func)
    signal.signal(signal.SIGINT, func)






# -*- coding: utf-8 -*-
"""
File Name:    ding_talk
Author:       Cone
Date:         2022/3/15
"""

import requests
import time
import hmac
import hashlib
import base64
import urllib.parse
import urllib
import random

url = 'https://oapi.dingtalk.com/robot/send'


class DingRobot:

    def __init__(self, name, token, secret):
        self.name = name
        self.token = token
        self.secret = secret

    def get_sign_timestamp(self):
        timestamp = str(round(time.time() * 1000))
        string_to_sign = '{}\n{}'.format(timestamp, self.secret)
        hmac_code = hmac.new(self.secret.encode('utf-8'), string_to_sign.encode('utf-8'), digestmod=hashlib.sha256).digest()
        sign = urllib.parse.quote_plus(base64.b64encode(hmac_code))
        return sign, timestamp

    def send_msg(self, content=None, msg_type="text"):
        # 每个机器人每分钟最多发送20条消息到群里，如果超过20条，会限流10分钟。
        sign, timestamp = self.get_sign_timestamp()
        data = {
            "msgtype": msg_type,
            msg_type: {"content": content}
        }
        params = {
            "access_token": self.token,
            "timestamp": timestamp,
            "sign": sign
        }
        response = requests.post(url, params=params, json=data)
        assert response.json()['errcode'] == 0, response.json()['errmsg']
        print("[%s]send message success, message is %s" % (self.name, data['text']))

    def __str__(self):
        return self.name

    __repr__ = __str__



# -*- coding: utf-8 -*-
"""
File Name:    pattern
Author:       Cone
Date:         2022/3/15
"""

from cone import patterns



@patterns.instance(1, b=2, c=3)
class E(metaclass=patterns.SingletonMeta):
    def __init__(self, a, b, c=1):
        self.a = a
        print(f"I'm instance of A({a}, {b}, c={c})")


@patterns.instance
class F(patterns.Singleton):
    def __init__(self):
        self.c = 13
        print("I'm instance of B()")



@patterns.singleton
class A:
    pass


class B(patterns.Singleton):
    def __init__(self, a, b):
        self.a = a
        self.b = b
        print(a, b)

class C(metaclass=patterns.SingletonMeta):
    def __init__(self, c):
        self.a = c
        print(c)

if __name__ == '__main__':

    # print(A() == A())
    # print(B(a=1, b=2) == B(a=3, b=2))
    # print(C(c='c') == C(c='d'))

    # print(E.a)
    # F

    def dd(a, *, c):
        print(a,  c)

    dd(1, b=1, c=1)
    print('')




# -*- coding: utf-8 -*-
"""
File Name:    os
Author:       Cone
Date:         2022/3/15
"""

from platform import system

# copy from you-get

def detect_os():
    """Detect operating system.
    """

    # Inspired by:
    # https://github.com/scivision/pybashutils/blob/78b7f2b339cb03b1c37df94015098bbe462f8526/pybashutils/windows_linux_detect.py

    syst = system().lower()
    os = 'unknown'

    if 'cygwin' in syst:
        os = 'cygwin'
    elif 'darwin' in syst:
        os = 'mac'
    elif 'linux' in syst:
        os = 'linux'
        # detect WSL https://github.com/Microsoft/BashOnWindows/issues/423
        try:
            with open('/proc/version', 'r') as f:
                if 'microsoft' in f.read().lower():
                    os = 'wsl'
        except: pass
    elif 'windows' in syst:
        os = 'windows'
    elif 'bsd' in syst:
        os = 'bsd'

    return os


if __name__ == '__main__':
    print("os is %s" % detect_os())






# -*- coding:utf-8 -*-

# author: Cone
# datetime: 2022/3/18 下午10:29
# software: PyCharm
import sys
from typing import Union


__all__ = [
    'hook_model_object'
]


def pop_sys_module(package, target_module):
    packages = {}
    t = None
    for k, v in sys.modules.items():
        if getattr(v, '__package__', None) == package:
            packages[k] = v
            if k == target_module:
                t = v
    for k in packages:
        sys.modules.pop(k)
    assert t is not None, '%s is not found in sys.modules' % target_module
    return t


def hook_model_object(source_obj: Union[object, str], target_obj):
    import importlib
    import inspect

    # get main.py stack
    frames = inspect.stack()
    frames.reverse()
    global_objects = frames[-2].frame.f_globals

    if type(source_obj) is str:
        module_str, obj_name = source_obj.rsplit('.', 1)
        module = importlib.import_module(module_str)
        source_obj = getattr(module, obj_name)
    obj_name = source_obj.__name__
    module_str = source_obj.__module__
    package = module_str.split('.')[0]
    sys_module = pop_sys_module(package, module_str)
    setattr(sys_module, obj_name, target_obj)
    sys.modules[module_str] = sys_module

    # global_objects = globals()
    module_parent, module_name = module_str.rsplit('.', 1)
    parent = importlib.import_module(module_parent)
    setattr(parent, module_name, sys_module)
    #
    for k, v in global_objects.items():
        if getattr(v, '__package__', None) == package:
            global_objects[k] = sys.modules[v.__name__]





# -*- coding: utf-8 -*-
"""
File Name:    singleton
Author:       Cone
Date:         2022/3/15
"""

_instances = {}


def _get_instance(cls, *args, **kwargs):
    global _instances
    if cls not in _instances:
        _instances[cls] = object.__new__(cls)
    instance = _instances[cls]
    if args or kwargs:
        instance.__init__(*args, **kwargs)
    return instance


def singleton(cls):
    def inner():
        return _get_instance(cls)
    return inner


class SingletonMeta(type):
    def __call__(cls, *args, **kwargs):
        return _get_instance(cls, *args, **kwargs)


class Singleton(metaclass=SingletonMeta):

    def __call__(self, *args, **kwargs):
        return _get_instance(self.__class__, *args, **kwargs)



# -*- coding:utf-8 -*-

# author: Cone
# datetime: 2022/10/20 下午5:20
# software: PyCharm
import json
import logging
try:
    from pymysql.err import IntegrityError
except ImportError:
    raise ImportError("please install pymysql first")


logger = logging.getLogger(__name__)


def build_insert_cmd(table, data: dict, option='insert'):
    columns = []
    values = []
    for key, value in data.items():
        if value is None:
            continue
        if isinstance(value, (list, dict)):
            data[key] = json.dumps(value, ensure_ascii=False)
        columns.append('`%s`' % key)
        values.append(f'%({key})s')
    return '{} into {}({}) values({})'.format(option, table, ','.join(columns), ','.join(values))


def insert_item(conn, cursor, item, table, option='insert') -> int:
    cmd = build_insert_cmd(table, item, option)
    try:
        cursor.execute(cmd, item)
        conn.commit()
        return 1
    except IntegrityError:
        logger.debug("insert duplicate: %s" % item)
        return 2
    except Exception as e:
        logger.exception("insert failed: %s, cmd is %s" % (e, cmd))
        return 0


def delete(conn, table, default_where=None, **where):
    children = [default_where] if default_where else []
    for key, value in where.items():
        if value is None:
            continue
        if isinstance(value, (str, int, float)):
            if '`' in key:
                children.append(f"{key} = '{value}'")
            else:
                children.append(f"`{key}` = '{value}'")
        else:
            raise Exception("invalid where value-type: %s" % type(value))
    assert children, "where can't be empty"
    cmd = 'delete from %s where %s' % (table, ' and '.join(children))
    with conn.cursor() as cursor:
        try:
            r = cursor.execute(cmd)
        except Exception as e:
            logger.exception("delete failed: %s, cmd is %s" % (e, cmd))
            return 0
    conn.commit()
    conn.close()
    logger.debug("delete %s rows from %s where %s" % (r, table, ' and '.join(children)))
    return r


def batch_insert(conn, table, items, option='insert'):
    s, f, d = 0, 0, 0
    with conn.cursor() as cursor:
        for item in items:
            cmd = build_insert_cmd(table, item, option=option)
            try:
                cursor.execute(cmd, item)
                s += 1
            except IntegrityError:
                d += 1
            except Exception as e:
                logger.exception("insert failed: %s, cmd is %s" % (e, cmd))
                f += 1
    conn.commit()
    conn.close()
    return s, f, d



# -*- coding: utf-8 -*-
"""
File Name:    curl
Author:       Cone
Date:         2022/3/15
"""
from cone.utils.network import curl



if __name__ == '__main__':
    a = curl.curl_to_request_kwargs('''
        curl 'https://www.toutiao.com/a7075223078497681923/?log_from=8783dc629c7aa_1647331371128' \
  -H 'authority: www.toutiao.com' \
  -H 'pragma: no-cache' \
  -H 'cache-control: no-cache' \
  -H 'sec-ch-ua: " Not A;Brand";v="99", "Chromium";v="99", "Google Chrome";v="99"' \
  -H 'sec-ch-ua-mobile: ?0' \
  -H 'sec-ch-ua-platform: "macOS"' \
  -H 'upgrade-insecure-requests: 1' \
  -H 'user-agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.51 Safari/537.36' \
  -H 'accept: text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9' \
  -H 'sec-fetch-site: same-origin' \
  -H 'sec-fetch-mode: navigate' \
  -H 'sec-fetch-user: ?1' \
  -H 'sec-fetch-dest: document' \
  -H 'referer: https://www.toutiao.com/' \
  -H 'accept-language: zh-CN,zh;q=0.9' \
  -H 'cookie: __ac_signature=_02B4Z6wo00f01D2JOJgAAIDDqV9jlsxXMFg9qTwAAG1t77; ttcid=62ce170b60824fb58f01260b5ee2efe014; tt_webid=7050003492509648421; csrftoken=268d5801b4f5c4c8918dae96f0ece274; _S_WIN_WH=1440_789; _S_DPR=2; _S_IPAD=0; passport_csrf_token_default=37e1e5a62575773f8ae57cfe03e906ab; passport_csrf_token=37e1e5a62575773f8ae57cfe03e906ab; _ga=GA1.2.1634366833.1644831025; local_city_cache=%E6%9D%AD%E5%B7%9E; s_v_web_id=verify_l0qgioj4_ZQpZcbQW_QNXG_4H3T_82fi_0Tny9mOBvf22; _tea_utm_cache_24=undefined; MONITOR_WEB_ID=34ce6355-b275-4995-a3a5-58097c34b98f; tt_scid=PDoxXb72oKUZ0CQBAtB8fpIp0LvA7tYlL2YsZWFCsYNNVBi0Jv2rhD87-rVU0Xd60ea2; ttwid=1%7C28Aw7TdVZbwJ4ayoEpMCQPQMLe2jVTl-2qCena6_E3k%7C1647331373%7C6edfffe1f9986db4e5302365e05128da65ca5e44c7bd92949aff311cbb51e5f1' \
  --compressed
    ''')
    import requests
    response = requests.request(**a)
    print(response.text)
    print(response.status_code)


# -*- coding: utf-8 -*-
"""
File Name:    signal
Author:       Cone
Date:         2022/3/21
"""


from cone.hooks import signal



def on_interrupt(signum, frame):
    print(signum, frame)
    exit(0)


signal.on_sys_interrupt(on_interrupt)

# while True:
#     import time
#     time.sleep(1)
#





class MysqlItemPipeline():
    pass


class SqlitePipeline():
    pass


class CSVPipline():
    pass


class JsonRecorder():
    pass


# -*- coding: utf-8 -*-
"""
File Name:    functional
Author:       Cone
Date:         2022/3/15
"""

from cone.utils.functional import cached_property, classproperty
from cone.utils import functional

class A:
    b = 1

    @cached_property
    def a(self):
        return self.b + 1

    @classproperty
    def c(cls):
        return cls.b + 4

class B:
    _a = 1

    @cached_property
    def a(self):
        return self._a + 2

a = A()
print(a.a)
print(a.a)

b = B()
print(b.a)
print(b.a)

print(A.c)
print(A().c)
import re

def _lazy_re_compile(regex, flags=0):
    """Lazily compile a regex with flags."""

    def _compile():
        # Compile the regex if it was not passed pre-compiled.
        if isinstance(regex, (str, bytes)):
            return re.compile(regex, flags)
        else:
            assert not flags, "flags must be empty if regex is passed pre-compiled"
            return regex

    return functional.SimpleLazyObject(_compile)


# a = functional.lazystr('dasdd')
# print(a)

a = _lazy_re_compile('redasd')

a.search('dasd')


# -*- coding:utf-8 -*-

# author: Cone
# datetime: 2022/3/20 上午11:19
# software: PyCharm
import requests
from pyquery import PyQuery
from lxml.html import etree
from cone.hooks import lib
from cone.utils.functional import cached_property


class Response(requests.Response):

    @cached_property
    def doc(self):
        return PyQuery(self.text)

    @cached_property
    def etree(self) -> etree._Element:
        return self.doc[0]

    def xpath(self, path):
        return self.etree.xpath(path)

# lib.hook_model_object('requests.Response', Response)
# #
#
# response: Response = requests.get(
#     'https://www.chinanews.com.cn/gn/2022/03-18/9706104.shtml',
# )
# response.encoding = 'utf8'
# print(response.doc("h1:first").text())
# print(response.xpath('//h1/text()')[0].strip())

# str.b = 1

print.a = 1

a = str(4)

print(a.b)


# print(requests.Response.doc)


# -*- coding:utf-8 -*-
# author: Cone
# datetime: 2022/11/2 09:42
# software: PyCharm
import os
import argparse

CURRENT_PATH = os.path.abspath(os.path.join(os.path.dirname(__file__), '../config'))
_CONFIG_PATH = []


class ConfigNotFoundError(FileNotFoundError):
    pass


def get_config_path(from_default=True, from_env=True, from_cmd=True, cmd_arg='config-path'):
    global _CONFIG_PATH
    if not _CONFIG_PATH:
        config_path = ''
        if from_env:
            config_path = os.getenv('CONFIG_PATH', '')
        elif from_cmd:
            import argparse
            parser = argparse.ArgumentParser()
            parser.add_argument('--%s' % cmd_arg, type=str, default='')
            args = parser.parse_known_args()[0]
            config_path = args.config_path
        if from_default:
            config_path = config_path + ',' + CURRENT_PATH
        for p in config_path.strip().split(','):
            if p:
                _CONFIG_PATH.append(os.path.abspath(p.strip()))
    return _CONFIG_PATH


class ConfigLoader:

    def __new__(cls, config_file: str, escape=True):
        if config_file.endswith('.json'):
            return JsonConfigLoader(config_file, escape=escape)
        elif config_file.endswith('.yaml'):
            return YamlConfigLoader(config_file, escape=escape)
        elif config_file.endswith('.py'):
            return PythonConfigLoader(config_file, escape=escape)
        raise NotImplementedError

    def __init__(self, config_file, escape=True):
        self.config_file = config_file
        self.escape = escape
        self._content = None
        self._text = None
        self._encoding = 'utf-8'

    @property
    def content(self):
        if not self._content:
            with open(self.config_file, 'rb', encoding=self._encoding) as f:
                self._content = f.read()
        return self._content

    @property
    def text(self):
        if not self._text:
            self._text = self.content.decode(self._encoding)
        return self._text

    def read(self):
        raise NotImplementedError


class JsonConfigLoader(ConfigLoader):

    def read(self):
        import json
        return json.loads(self.content)


class YamlConfigLoader(ConfigLoader):
    def read(self):
        try:
            import yaml
        except ImportError:
            raise ImportError('Please install PyYAML')
        return yaml.load(self.text, Loader=yaml.FullLoader)


class PythonConfigLoader(ConfigLoader):
    def read(self):
        import importlib.util
        spec = importlib.util.spec_from_file_location('config', self.config_file)
        config = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(config)
        return config


def translate_config(config):
    if isinstance(config, dict):
        for k, v in config.copy().items():
            if '-' in k:
                config[k.replace('-', '_')] = v
                del config[k]
            if isinstance(v, dict):
                translate_config(v)
    elif isinstance(config, list):
        for v in config:
            translate_config(v)


def _load_config(name_or_path: str, suffixes=None, escape=True):
    suffixes = suffixes or ['.json']
    with_suffix = name_or_path.split(os.sep)[-1].split('.')[-1] in suffixes
    for p in get_config_path():
        if with_suffix:
            config_file = os.path.join(p, name_or_path)
            if os.path.exists(config_file):
                return ConfigLoader(config_file, escape=escape)
        else:
            for suffix in suffixes:
                config_file = os.path.join(p, '%s%s' % (name_or_path, suffix))
                if os.path.exists(config_file):
                    return ConfigLoader(config_file, escape=escape)
    raise ConfigNotFoundError("config(%s[%s]) not found in %s" % (name_or_path, '|'.join(suffixes), get_config_path()))


def load_config(name_or_path=None, cmd_arg=None, suffixes=None, escape=True):
    assert name_or_path or cmd_arg, "name_or_path or cmd_arg must be set"
    if cmd_arg is not None:
        parser = argparse.ArgumentParser()
        parser.add_argument('--%s' % cmd_arg, type=str)
        args = parser.parse_known_args()[0]
        if args.cmd_arg:
            name_or_path = args.cmd_arg
    return _load_config(name_or_path, suffixes=suffixes, escape=escape)


if __name__ == '__main__':

    load_config('test')


# -*- coding:utf-8 -*-

# author: Cone
# datetime: 2022/3/15 下午9:22
# software: PyCharm
from cone.communication import DingRobot

robots = [
    DingRobot(name='裘一', token='deca7812002f387023c9cc04b77d0a8c9ed1a4f703f19cd16e431806edb8c9bd', secret='SEC1692e6f5ba22183a6c9cc41a38bbb8e03a5537c90fb6a867b7e34f9ed27ad0f3'),
    DingRobot(name='不上', token='18b48b2ddf7e13944fafb4a25f3129e50b70bd8f44e6b4704b5e93799f618505', secret='SEC551cb984408051fb0544bbca1c21b6c14abd5da9fd1f0c197ce1408f65e40858')
]


for robot in robots:
    robot.send_msg(content="Hello, I am %s" % robot.name)




# -*- coding: utf-8 -*-
"""
File Name:    exception
Author:       Cone
Date:         2022/3/15
"""

from cone.communication import ding_talk
from cone.hooks import exception

def a(*args):
    ding_talk.DingRobot(
        name='裘一',
        token='deca7812002f387023c9cc04b77d0a8c9ed1a4f703f19cd16e431806edb8c9bd',
        secret='SEC1692e6f5ba22183a6c9cc41a38bbb8e03a5537c90fb6a867b7e34f9ed27ad0f3'
    ).send_msg("args is %s" % str(args))

exception.setSysExceptHook(a)

1/0


# -*- coding:utf-8 -*-

# author: Cone
# datetime: 2022/3/20 下午4:29
# software: PyCharm
import hashlib


def get_md5(string: str):
    m = hashlib.md5()
    m.update(string.encode(encoding='utf-8'))
    return m.hexdigest()


# -*- coding: utf-8 -*-
"""
File Name:    url
Author:       Cone
Date:         2022/3/15
"""
import re
from urllib.parse import urlparse

from urllib3.util import parse_url


def add_http_if_no_scheme(url):
    """Add http as the default scheme if it is missing from the url."""
    match = re.match(r"^\w+://", url, flags=re.I)
    if not match:
        parts = urlparse(url)
        scheme = "http:" if parts.netloc else "http://"
        url = scheme + url

    return url


def url_is_from_any_domain(url, domains):
    """Return True if the url belongs to any of the given domains"""
    host = parse_url(url).netloc.lower()
    if not host:
        return False
    domains = [d.lower() for d in domains]
    return any((host == d) or (host.endswith(f'.{d}')) for d in domains)



# -*- coding: utf-8 -*-
"""
File Name:    exception
Author:       Cone
Date:         2022/3/15
"""
import sys

sys_excepthook = sys.excepthook


def setSysExceptHook(onExcept):
    def _sys_excepthook(excType, excValue, tb):
        sys_excepthook(excType, excValue, tb)
        onExcept(excType, excValue, tb)
    sys.excepthook = _sys_excepthook



# -*- coding:utf-8 -*-

# author: Cone
# datetime: 2020-01-10 17:19
# software: PyCharm
from inspect import isclass


def instance(*args, **kwargs):
    def params_instance(cls):
        return cls(*args, **kwargs)  # 带参数的实例
    if args and isclass(args[0]):
        # bug: function args[0] cant be a class type
        return args[0]()
    return params_instance




import os
import sys
import pathlib
import importlib
import inspect
from typing import Union

WORKING_DIR = os.getcwd()
WORKING_DIR_SPLIT = WORKING_DIR.split(os.sep)


class ClassManager(dict):
    _mapping = {}

    @staticmethod
    def to_path_list(path=None):
        if path is None:
            path = []
        elif isinstance(path, str):
            path = path.split(',')
        assert isinstance(path, list), "path must be list of string, but got %s" % type(path)
        return path

    def __new__(cls, path=None, name=None, **kwargs):
        path = cls.to_path_list(path)
        key = ",".join(path) or name
        assert key, "path or name must be provided"
        if key not in cls._mapping:
            cls._mapping[key] = super(ClassManager, cls).__new__(cls)
        return cls._mapping[key]

    def __init__(self, *, path=None, name=None, unique_keys=None):
        super(ClassManager, self).__init__()
        assert unique_keys and isinstance(unique_keys, (str, list, tuple)), \
            "unique_keys must be string or list of string, but got %s" % type(unique_keys)
        path = self.to_path_list(path)
        self.path = [x.replace(os.sep, '.') for x in path]
        if isinstance(unique_keys, str):
            unique_keys = [unique_keys]
        self.unique_keys = unique_keys
        self.name = name
        self._loaded = False

    def _is_manageable_class(self, cls):
        return getattr(cls, '__manager__', None) == self

    @staticmethod
    def _is_python_module(file: pathlib.Path):
        return file.suffix == '.py' and (not file.stem.startswith('_'))

    @staticmethod
    def _is_python_package(path: pathlib.Path):
        return path.is_dir() and (path / '__init__.py').exists()

    @staticmethod
    def find_common_prefix(path1, path2):
        for i in range(min(len(path1), len(path2))):
            if path1[i] != path2[i]:
                return path1[:i]
        return path1[:min(len(path1), len(path2))]

    @staticmethod
    def _load_module(module_path):
        return importlib.import_module(module_path)

    def _load_package(self, module_path: str):
        try:
            module = self._load_module(module_path)
        except Exception as e:
            raise ImportError("Failed to load %s: %s" % (module_path, e))
        module_file = pathlib.Path(module.__file__)
        if module_file.name == "__init__.py":
            package = module_file.parent
            for p in package.glob('*'):
                if self._is_python_package(p) or self._is_python_module(p):
                    self._load_package(module_path + '.' + p.stem)

    def __call__(self, *, is_registry=True, generator=None, overwritable=False, **kwargs):
        is_registry = is_registry or generator is not None
        if is_registry:
            for k, v in kwargs.items():
                if k not in self.unique_keys:
                    is_registry = False
                    break
        if not is_registry:
            cls = self.find(**kwargs)
            spec = inspect.getfullargspec(cls)
            if spec.varkw is None:
                args = spec.args
                for k in self.unique_keys:
                    if k not in args:
                        kwargs.pop(k)
            return cls(**kwargs)
        else:
            return self.register(generator=generator, overwritable=overwritable, **kwargs)

    def register(self, generator=None, overwritable=False, **kwargs):
        def wrapper(cls):
            if generator:
                assert callable(generator), "generator must be callable"
                for keys in generator():
                    if isinstance(keys, str):
                        keys = [keys]
                    self._add_class(cls, overwritable=overwritable, generated=True,
                                    **dict(zip(self.unique_keys, keys)))
            else:
                self._add_class(cls, **kwargs)
            return cls

        return wrapper

    def register_from(self, path):
        if self._loaded:
            self._load_package(path)
        else:
            self.path.append(path)

    def _gen_key(self, **kwargs):
        if len(self.unique_keys) == 1:
            return kwargs[self.unique_keys[0]]
        return tuple(kwargs[key] for key in self.unique_keys)

    def _add_class(self, cls, overwritable=False, generated=False, **kwargs):
        for k in self.unique_keys:
            v = kwargs.get(k, None)
            if v is None:
                v = getattr(cls, k, None)
                kwargs[k] = v
            assert v is not None, "class %s must have %s" % (cls.__name__, k)
        unique_key = self._gen_key(**kwargs)
        exists = super(ClassManager, self).get(unique_key, None)
        if exists and not getattr(exists, '__overwritable__', False):
            raise KeyError("class %s already exists and is not overwritable" % unique_key)
        setattr(cls, '__overwritable__', overwritable)
        setattr(cls, '__manager__', self)
        if generated:
            setattr(cls, '__generated__', True)
            setattr(cls, '__generated_args__', kwargs)
        else:
            for key, value in kwargs.items():
                if key in self.unique_keys:
                    setattr(cls, key, value)
        self[unique_key] = cls

    def _ensure_loaded(self):
        if not self._loaded:
            for path in self.path:
                self._load_package(path)
            self._loaded = True

    def __getitem__(self, key):
        self._ensure_loaded()
        cls = super(ClassManager, self).__getitem__(key)
        if getattr(cls, '__generated__', False):
            for key, value in getattr(cls, '__generated_args__', {}).items():
                if key in self.unique_keys:
                    setattr(cls, key, value)
        return cls

    def __iter__(self):
        self._ensure_loaded()
        return super(ClassManager, self).__iter__()

    def keys(self):
        self._ensure_loaded()
        return super(ClassManager, self).keys()

    def values(self):
        self._ensure_loaded()
        return super(ClassManager, self).values()

    def items(self):
        self._ensure_loaded()
        return super(ClassManager, self).items()

    def find(self, **kwargs):
        return self[self._gen_key(**kwargs)]

    def __str__(self):
        return "ClassManager(path=%s, num=%s)" % (self.path, len(self))



# -*- coding:utf-8 -*-

# author: Cone
# datetime: 2022/3/20 下午4:31
# software: PyCharm

# from Crypto.Cipher import AES
# from Crypto import Random
# import base64
# import msvcrt, sys


class AESEncrypt(object):
    def __init__(self, key):
        self.key = key

    def format_key(cls, key):
        sub_key = '*' * (16 - len(key))
        key += sub_key
        return key

    def padding(cls, s):
        pad = lambda s: s + (AES.block_size - len(s) % AES.block_size) * chr(0) #chr(AES.block_size - len(s) % AES.block_size)
        return pad(s)

    def unpadding(cls, s):
        # unpad = lambda s : s[:-ord(s[len(s)-1:])]
        unpad = lambda s: s.rstrip(chr(0))
        return unpad(s)

    def encrypt(cls, text, key=None):
        # key = cls.key if not key else cls.format_key(key)
        # text = AESEncrypter.padding(text)
        # iv = Random.new().read(AES.block_size)
        # cryptor = AES.new(key, AES.MODE_CBC, iv)
        # result = base64.b64encode(iv + cryptor.encrypt(text))
        return result.decode()

    def decrypt(cls, text, key=None):
        pass


def encrypt(key, ):
    return AESEncrypt(key).encrypt()


from setuptools import setup, find_packages

setup(
    name='py-cone',
    packages=find_packages(),
    version='1.0.9',
    # install_requires=[
    # ],
    # extras_require={
    # },
    license='MIT',
    url='https://github.com/cone387/cone-python',
    python_requires='>=3.7, <4',
)
